@@ -9,23 +9,37 @@ import lxml.html
 from copy import deepcopy
 from urlparse import urljoin
 
-def fromstring(context, parser=None):
+def fromstring(context, parser=None, custom_parser=None):
     """use html parser if we don't have clean xml
     """
-    if parser == None:
-        try:
-            return [etree.fromstring(context)]
-        except etree.XMLSyntaxError:
-            return [lxml.html.fromstring(context)]
-    elif parser == 'xml':
-        return [etree.fromstring(context)]
-    elif parser == 'html':
-        return [lxml.html.fromstring(context)]
-    elif parser == 'html_fragments':
-        return lxml.html.fragments_fromstring(context)
+    if custom_parser is None:
+        if parser is None:
+            try:
+                return [etree.fromstring(context)]
+            except etree.XMLSyntaxError:
+                return [lxml.html.fromstring(context)]
+        
+        elif parser == 'xml':
+            custom_parser = etree.fromstring
+        elif parser == 'html':
+            custom_parser = lxml.html.fromstring
+        elif parser == 'soup':
+            from  lxml.html import soupparser
+            custom_parser = lxml.html.soupparser.fromstring
+        elif parser == 'html_fragments':
+            custom_parser = lxml.html.fragments_fromstring
+        else:
+            ValueError('No such parser: "%s"' % parser)
+    
+    result = custom_parser(context)
+    if type(result) is list:
+        return result
     else:
-        ValueError('No such parser: "%s"' % parser)
+        return [result]
 
+def callback(func, *args):
+    return func(*args[:func.func_code.co_argcount])
+        
 class NoDefault(object):
     def __repr__(self):
         """clean representation in Sphinx"""
@@ -72,9 +86,11 @@ class PyQuery(list):
         self.parser = kwargs.get('parser', None)
         if 'parser' in kwargs:
             del kwargs['parser']
-        if not kwargs and len(args) == 1 and isinstance(args[0], basestring) \
+        if len(args) >= 1 and isinstance(args[0], basestring) \
            and args[0].startswith('http://'):
-            kwargs = {'url': args[0]}
+            kwargs['url'] = args[0]
+            if len(args) >= 2:
+                kwargs['data'] = args[1]
             args = []
 
         if 'parent' in kwargs:
@@ -93,7 +109,21 @@ class PyQuery(list):
                     html = opener(url)
                 else:
                     from urllib2 import urlopen
-                    html = urlopen(url).read()
+                    from urllib import urlencode
+                    method = kwargs.get('method')
+                    data = kwargs.get('data')
+                    if type(data) in (dict, list, tuple):
+                        data = urlencode(data)
+                        
+                    if isinstance(method, basestring) and method.lower() == 'get' and data:
+                        if '?' not in url:
+                            url += '?'
+                        elif url[-1] not in ('?', '&'):
+                            url += '&'
+                        url += data
+                        data = None
+                    
+                    html = urlopen(url, data).read()
                 self._base_url = url
             else:
                 raise ValueError('Invalid keyword arguments %s' % kwargs)
@@ -143,7 +173,9 @@ class PyQuery(list):
         length = len(args)
         if length == 0:
             raise ValueError('You must provide at least a selector')
-        if len(args) == 1 and not args[0].startswith('<'):
+        if args[0] == '':
+            return self.__class__([])
+        if len(args) == 1 and isinstance(args[0], str) and not args[0].startswith('<'):
             args += (self,)
         result = self.__class__(*args, **dict(parent=self))
         return result
@@ -347,10 +379,11 @@ class PyQuery(list):
             try:
                 for i, this in enumerate(self):
                     selector.func_globals['this'] = this
-                    if selector(i):
+                    if callback(selector, i):
                         elements.append(this)
             finally:
-                del selector.func_globals['this']
+                if 'this' in selector.func_globals:
+                    del selector.func_globals['this']
             return self.__class__(elements, **dict(parent=self))
 
     def not_(self, selector):
@@ -409,8 +442,14 @@ class PyQuery(list):
     def each(self, func):
         """apply func on each nodes
         """
-        for e in self:
-            func(self.__class__([e]))
+        try:
+            for i, element in enumerate(self):
+                func.func_globals['this'] = element
+                if callback(func, i, element) == False:
+                    break
+        finally:
+            if 'this' in func.func_globals:
+                del func.func_globals['this']
         return self
 
     def map(self, func):
@@ -428,19 +467,24 @@ class PyQuery(list):
 
             >>> d('p').map(lambda i, e: PyQuery(this).text().split())
             ['Hi', 'there', 'Bye']
+            
+            Added this 
+
+
         """
         items = []
         try:
             for i, element in enumerate(self):
                 func.func_globals['this'] = element
-                result = func(i, element)
+                result = callback(func, i, element)
                 if result is not None:
                     if not isinstance(result, list):
                         items.append(result)
                     else:
                         items.extend(result)
         finally:
-            del func.func_globals['this']
+            if 'this' in func.func_globals:
+                del func.func_globals['this']
         return self.__class__(items, **dict(parent=self))
 
     @property
@@ -659,15 +703,15 @@ class PyQuery(list):
 
         Get the text value::
 
-            >>> doc = PyQuery('<div><span>toto</span></div>')
-            >>> print doc.html()
+            >>> d = PyQuery('<div><span>toto</span></div>')
+            >>> print d.html()
             <span>toto</span>
 
         Set the text value::
 
-            >>> doc.html('<span>Youhou !</span>')
+            >>> d.html('<span>Youhou !</span>')
             [<div>]
-            >>> print doc
+            >>> print d
             <div><span>Youhou !</span></div>
         """
         if value is no_default:
@@ -697,6 +741,28 @@ class PyQuery(list):
                 tag.tail = root.tail
         return self
 
+    def outerHtml(self):
+        """Get the html representation of the first selected element.
+
+            >>> d = PyQuery('<div><span class="red">toto</span> rocks</div>')
+            >>> print d('span')
+            <span class="red">toto</span> rocks
+            >>> print d('span').outerHtml()
+            <span class="red">toto</span>
+
+            >>> S = PyQuery('<p>Only <b>me</b> & myself</p>')
+            >>> S('b').outerHtml()
+            '<b>me</b>'
+        """
+        
+        if not self:
+            return None
+        e0 = self[0]
+        if e0.tail:
+            e0 = deepcopy(e0)
+            e0.tail = ''
+        return lxml.html.tostring(e0)
+        
     def text(self, value=no_default):
         """Get or set the text representation of sub nodes.
 
@@ -977,21 +1043,50 @@ class PyQuery(list):
 
     def remove(self, expr=no_default):
         """remove nodes
+        
+        >>> d = PyQuery('<div>Maybe <em>she</em> does <strong>NOT</strong> know</div>')
+        >>> d('strong').remove()
+        [<strong>]
+        >>> print d
+        <div>Maybe <em>she</em> does   know</div>
         """
         if expr is no_default:
             for tag in self:
                 parent = tag.getparent()
                 if parent is not None:
                     if tag.tail:
-                        if not parent.text:
-                            parent.text = ''
-                        parent.text += ' ' + tag.tail
+                        prev = tag.getprevious()
+                        if prev is None:
+                            if not parent.text:
+                                parent.text = ''
+                            parent.text += ' ' + tag.tail
+                        else:
+                            if not prev.tail:
+                                prev.tail = ''
+                            prev.tail += ' ' + tag.tail
                     parent.remove(tag)
         else:
             results = self.__class__(expr, self)
             results.remove()
         return self
 
+    class Fn(object):
+        """Hook for defining custom funcion (like the jQuery.fn)
+        
+        >>> PyQuery.fn.listOuterHtml = lambda: this.map(lambda i, el: PyQuery(this).outerHtml())
+        >>> S = PyQuery('<ol>   <li>Coffee</li>   <li>Tea</li>   <li>Milk</li>   </ol>')
+        >>> S('li').listOuterHtml()
+        ['<li>Coffee</li>', '<li>Tea</li>', '<li>Milk</li>']
+        
+        """
+        def __setattr__(self, name, func):
+            def fn(self, *args):
+                func.func_globals['this'] = self
+                return func(*args)
+            fn.__name__ = name
+            setattr(PyQuery, name, fn)
+    fn = Fn()
+    
     #####################################################
     # Additional methods that are not in the jQuery API #
     #####################################################
@@ -1014,6 +1109,5 @@ class PyQuery(list):
                 raise ValueError('You need a base URL to make your links'
                  'absolute. It can be provided by the base_url parameter.')
 
-        self('a').each(lambda a:
-                       a.attr('href', urljoin(base_url, a.attr('href'))))
+        self('a').each(lambda: self(this).attr('href', urljoin(base_url, self(this).attr('href'))))
         return self
