@@ -21,7 +21,7 @@ try:
     conn = httplib.HTTPConnection("pyquery.org:80")
     conn.request("GET", "/")
     response = conn.getresponse()
-except socket.timeout:
+except (socket.timeout, socket.error):
     GOT_NET=False
 else:
     GOT_NET=True
@@ -201,6 +201,11 @@ class TestSelector(unittest.TestCase):
         assert len(e(":parent")) == 2
         assert len(e(":empty")) == 6
         assert len(e(":contains('Heading')")) == 6
+        
+    def test_on_the_fly_dom_creation(self):
+        e = self.klass(self.html)
+        assert e('<p>Hello world</p>').text() == 'Hello world'
+        assert e('').text() == None
 
 class TestTraversal(unittest.TestCase):
     klass = pq
@@ -217,6 +222,9 @@ class TestTraversal(unittest.TestCase):
         assert len(self.klass('div', self.html).filter('.node3')) == 1
         assert len(self.klass('div', self.html).filter('#node2')) == 1
         assert len(self.klass('div', self.html).filter(lambda i: i == 0)) == 1
+        
+        d = pq('<p>Hello <b>warming</b> world</p>')
+        self.assertEqual(d('strong').filter(lambda el: True), [])
 
     def test_not(self):
         assert len(self.klass('div', self.html).not_('.node3')) == 1
@@ -232,18 +240,25 @@ class TestTraversal(unittest.TestCase):
 
     def test_each(self):
         doc = self.klass(self.html)
-        doc('span').each(lambda e: e.wrap("<em></em>"))
+        doc('span').each(lambda: doc(this).wrap("<em></em>"))
         assert len(doc('em')) == 3
 
     def test_map(self):
         def ids_minus_one(i, elem):
             return int(self.klass(elem).attr('id')[-1]) - 1
         assert self.klass('div', self.html).map(ids_minus_one) == [0, 1]
+        
+        d = pq('<p>Hello <b>warming</b> world</p>')
+        self.assertEqual(d('strong').map(lambda i,el: pq(this).text()), [])
 
     def test_end(self):
         assert len(self.klass('div', self.html).find('span').end()) == 2
         assert len(self.klass('#node2', self.html).find('span').end()) == 1
 
+    def test_closest(self):
+        assert len(self.klass('#node1 span', self.html).closest('body')) == 1
+        assert self.klass('#node2', self.html).closest('.node3').attr('id') == 'node2'
+        assert self.klass('.node3', self.html).closest('form') == []
 
 class TestOpener(unittest.TestCase):
 
@@ -254,6 +269,23 @@ class TestOpener(unittest.TestCase):
         doc = pq(url='http://example.com', opener=opener)
         assert len(doc('.node')) == 1, doc
 
+class TestCallback(unittest.TestCase):
+    html = """
+        <ol>
+            <li>Coffee</li>
+            <li>Tea</li>
+            <li>Milk</li>
+        </ol>
+    """
+    
+    def test_S_this_inside_callback(self):
+        S = pq(self.html)
+        self.assertEqual(S('li').map(lambda i, el: S(this).html()), ['Coffee', 'Tea', 'Milk'])
+        
+    def test_parameterless_callback(self):
+        S = pq(self.html)
+        self.assertEqual(S('li').map(lambda: S(this).html()), ['Coffee', 'Tea', 'Milk'])
+        
 def application(environ, start_response):
     req = Request(environ)
     response = Response()
@@ -275,7 +307,7 @@ class TestAjaxSelector(TestSelector):
     def test_proxy(self):
         e = self.klass([])
         val = e.get('http://pyquery.org/')
-        assert len(val('div.document')) == 1, (val.response, val)
+        assert len(val('body')) == 1, (str(val.response), val)
 
     def test_get(self):
         e = self.klass(app=application)
@@ -321,6 +353,58 @@ class TestManipulating(unittest.TestCase):
         val = d('a:last').html()
         assert val == ' My link text 2', repr(val)
 
+class TestHTMLParser(unittest.TestCase):
+    xml = "<div>I'm valid XML</div>"
+    html = '''
+    <div class="portlet">
+      <a href="/toto">TestimageMy link text</a>
+      <a href="/toto2">imageMy link text 2</a>
+      Behind you, a three-headed HTML&dash;Entity!
+    </div>
+    '''
+    def test_parser_persistance(self):
+        d = pq(self.xml, parser='xml')
+        self.assertRaises(etree.XMLSyntaxError, lambda: d.after(self.html))
+        d = pq(self.xml, parser='html')
+        d.after(self.html) # this should not fail
+        
+        d = pq('<meta><head><title>Hello</head><body onload=crash()>Hi all<p>', parser='soup')
+        self.assertEqual(str(d), '<html><meta/><head><title>Hello</title></head><body onload="crash()">Hi all<p/></body></html>')
+
+    def test_replaceWith(self):
+        expected = '''<div class="portlet">
+      <a href="/toto">TestimageMy link text</a>
+      <a href="/toto2">imageMy link text 2</a>
+      Behind you, a three-headed HTML&amp;dash;Entity!
+    </div>'''
+        d = pq(self.html)
+        d('img').replaceWith('image')
+        val = d.__html__()
+        assert val == expected, (repr(val), repr(expected))
+
+    def test_replaceWith_with_function(self):
+        expected = '''<div class="portlet">
+      TestimageMy link text
+      imageMy link text 2
+      Behind you, a three-headed HTML&amp;dash;Entity!
+    </div>'''
+        d = pq(self.html)
+        d('a').replaceWith(lambda i, e: pq(e).html())
+        val = d.__html__()
+        assert val == expected, (repr(val), repr(expected))
+
+class TestWebScrapping(unittest.TestCase):
+    @with_net
+    def test_get(self):
+        d = pq('http://www.theonion.com/search/', {'q': 'inconsistency'}, method='get')
+        self.assertEqual(d('input [name=q]:last').val(), 'inconsistency')
+        self.assertEqual(d('.news-in-brief h3').text(), 'Slight Inconsistency Found In Bible')
+    
+    @with_net
+    def test_post(self):
+        d = pq('http://www.theonion.com/search/', {'q': 'inconsistency'}, method='post')
+        self.assertEqual(d('input [name=q]:last').val(), '') # the onion does not search on post
+
 if __name__ == '__main__':
     fails, total = unittest.main()
     if fails == 0:
